#!/bin/bash
# Copyright 2008 - 2010, 2013, 2014 Michal Žejdl
# called from collector.sh
# it reads SNMP FDB of given node and fills the MySQL table fdb
# input: node (arg), checked.hasfdb (sql)
# dependence: dirname, mysql (sys table), cut, snmpwalk, gawk

test $# -eq 1 || exit
node=$1

confdir=$(dirname $0)/..
config=$confdir/defaults.sh
test -f $config || exit
. $config

# use previous check of fdb
# distinct dns will be obsolete after implementing a new unique node db schema
test "$(mysql -e "select distinct dns from checked where dns = '$node' and hasfdb = 'y'" $select_opts)" = "$node" || exit

# bridge FDB - ports (with MAC addresses as oids)
oid=".1.3.6.1.2.1.17.4.3.1.2"

# uncomment after debug
exec 3> $mysqlwpipe

# get sysObjectID (SNMPv2-SMI::enterprises.*) from database
sysoid=$(mysql -e "select trim(leading 'SNMPv2-SMI::enterprises.' from sysObjectID) from sys where host = '$node'" $select_opts)

vendor=$(echo $sysoid | cut -d '.' -f1)
vlans=""

case $vendor in
	9 )
		# Cisco
		vlans=$(snmpwalk -Cc -m '' -v 1 -c $snmprcommunity -Onq $node .1.3.6.1.4.1.9.9.46.1.3.1.1.2 2>> $logdir/fdb-snmpwalk.err | gawk -F[.[:space:]] '/^\.1\.3\.6\.1\.4\.1\.9\.9\.46\.1\.3\.1\.1\.2\.1\.[[:digit:]]+[[:space:]]1$/ {print $17}')
		;;
	43 )
		# 3Com
		case $sysoid in
			43.10.27.4.1.2.1 | 43.10.27.4.1.2.2 )
				# Switch 1100 or 3300
				vlans=$(snmpwalk -Cc -m '' -v 1 -c $snmprcommunity -Onq $node .1.3.6.1.4.1.43.10.1.14.1.1 2>> $logdir/fdb-snmpwalk.err | gawk -F[.[:space:]] '/^\.1\.3\.6\.1\.4\.1\.43\.10\.1\.14\.1\.1\.1\.2\.[[:digit:]]+[[:space:]][[:digit:]]+$/ {print $16}')
				;;
		esac
		;;
esac

if test -z "$vlans"; then
	communities=$snmprcommunity
else
	# to get FDB by iterating <community name>@<VLAN number>
	communities=$(for vlan in $vlans; do echo $snmprcommunity@$vlan; done)
fi

for community in $communities; do
	# snmpwalk - numeric oids, only values (integer), TO to log file
	# gawk - enable intervals in regexps for ver. 3, input checking
	snmpwalk -Cc -m '' -v 1 -c $community -Onq $node $oid 2>> $logdir/fdb-snmpwalk.err | gawk -F[.[:space:]] -v node=$node --re-interval '/^\.1\.3\.6\.1\.2\.1\.17\.4\.3\.1\.2(\.[[:digit:]]{1,3}){6}[[:space:]][[:digit:]]+$/ {printf "insert fdb values (\47%s\47, %d, \47%02x:%02x:%02x:%02x:%02x:%02x\47, unix_timestamp()) on duplicate key update port = %d, time = unix_timestamp();\n", node, $19, $13, $14, $15, $16, $17, $18, $19; fflush()}' >&3
done
