#!/bin/bash
# Copyright 2016, 2017 Michal Žejdl
# snmpget enterprises.14988 OID(s) and store to RRD and SQL

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
. $config

functions=$coredir/snmp/functions.sh
test -f $functions || exit
. $functions

address=$1
agent=$2

# a new file descriptor to prevent pipe from closing
exec 3> $mysqlwpipe

# group write permissions for rrdcached
umask 002

rrdstep=$snmpinterval
rrdheartbeat=$((2 * rrdstep))
rraxff=0.5
rrathirtym=$((30 * 60 / rrdstep))
rratwoh=$((4 * rrathirtym))
rraday=$((12 * rratwoh))
rraweek=$((7 * rraday))
rrarows=600

rrddestdir=$rrddir/snmpagent/$agent/MikroTik
test -d "$rrddestdir" || mkdir -p "$rrddestdir" || exit

# debugging
#echo "$(date '+%F %X') $(basename $0) address $address agent $agent"


# mtxrWlStatTable

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'
# MikroTik doesn't have mandatory values here so snmpwalk is a must
# no retry (-r 0), do not resend request if OID is missing (-Cf)
result=($(snmpwalk -Cc -m '' -v 1 -r 0 -c $snmprcommunity -Onqt $address \
.1.3.6.1.4.1.14988.1.1.1.1.1.4 \
))

snmpget_status=$?
#test $snmpget_status -eq 0 || exit

IFS=$oldifs


# Undefined is better than 0 for absolute values
mtxrWlStatStrength=U

# ugly sanitizing
for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do
	case ${result[oid_row]::30} in
	.1.3.6.1.4.1.14988.1.1.1.1.1.4)
		mtxrWlStatStrength=${result[$oid_row]#.1.3.6.1.4.1.14988.1.1.1.1.1.4.* }
		is_number "$mtxrWlStatStrength" || mtxrWlStatStrength=U
		;;
	esac
done

# debugging
#echo "$(date '+%F %X') $(basename $0) mtxrWlStatStrength=$mtxrWlStatStrength"

# check if we got at last one value
if test $mtxrWlStatStrength != U
then
	rrdfile="$rrddestdir/mtxrWlStat.rrd"

	# --start is not necessary
	test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
	DS:Strength:GAUGE:$rrdheartbeat:U:U \
	RRA:AVERAGE:$rraxff:1:$rrarows \
	RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
	RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
	RRA:AVERAGE:$rraxff:$rraday:$rrarows \
	RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
	RRA:MIN:$rraxff:1:$rrarows \
	RRA:MIN:$rraxff:$rrathirtym:$rrarows \
	RRA:MIN:$rraxff:$rratwoh:$rrarows \
	RRA:MIN:$rraxff:$rraday:$rrarows \
	RRA:MIN:$rraxff:$rraweek:$rrarows \
	RRA:MAX:$rraxff:1:$rrarows \
	RRA:MAX:$rraxff:$rrathirtym:$rrarows \
	RRA:MAX:$rraxff:$rratwoh:$rrarows \
	RRA:MAX:$rraxff:$rraday:$rrarows \
	RRA:MAX:$rraxff:$rraweek:$rrarows \
	RRA:LAST:$rraxff:1:$rrarows \
	RRA:LAST:$rraxff:$rrathirtym:$rrarows \
	RRA:LAST:$rraxff:$rratwoh:$rrarows \
	RRA:LAST:$rraxff:$rraday:$rrarows \
	RRA:LAST:$rraxff:$rraweek:$rrarows

	rrdtool update "$rrdfile" $rrdcached N:\
$mtxrWlStatStrength

fi


# mtxrWlApTable

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'
# MikroTik doesn't have mandatory values here so snmpwalk is a must
# no retry (-r 0), do not resend request if OID is missing (-Cf)
result=($(snmpwalk -Cc -m '' -v 1 -r 0 -c $snmprcommunity -Onqt $address \
.1.3.6.1.4.1.14988.1.1.1.3.1.6 \
))

snmpget_status=$?
#test $snmpget_status -eq 0 || exit

IFS=$oldifs


# Undefined is better than 0 for absolute values
mtxrWlApClientCount=U

# ugly sanitizing
for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do
	case ${result[oid_row]::30} in
	.1.3.6.1.4.1.14988.1.1.1.3.1.6)
		mtxrWlApClientCount=${result[$oid_row]#.1.3.6.1.4.1.14988.1.1.1.3.1.6.* }
		is_number "$mtxrWlApClientCount" || mtxrWlApClientCount=U
		;;
	esac
done

# check if we got at last one value
if test $mtxrWlApClientCount != U
then
	rrdfile="$rrddestdir/mtxrWlAp.rrd"

	# --start is not necessary
	test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
	DS:ClientCount:GAUGE:$rrdheartbeat:0:U \
	RRA:AVERAGE:$rraxff:1:$rrarows \
	RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
	RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
	RRA:AVERAGE:$rraxff:$rraday:$rrarows \
	RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
	RRA:MIN:$rraxff:1:$rrarows \
	RRA:MIN:$rraxff:$rrathirtym:$rrarows \
	RRA:MIN:$rraxff:$rratwoh:$rrarows \
	RRA:MIN:$rraxff:$rraday:$rrarows \
	RRA:MIN:$rraxff:$rraweek:$rrarows \
	RRA:MAX:$rraxff:1:$rrarows \
	RRA:MAX:$rraxff:$rrathirtym:$rrarows \
	RRA:MAX:$rraxff:$rratwoh:$rrarows \
	RRA:MAX:$rraxff:$rraday:$rrarows \
	RRA:MAX:$rraxff:$rraweek:$rrarows \
	RRA:LAST:$rraxff:1:$rrarows \
	RRA:LAST:$rraxff:$rrathirtym:$rrarows \
	RRA:LAST:$rraxff:$rratwoh:$rrarows \
	RRA:LAST:$rraxff:$rraday:$rrarows \
	RRA:LAST:$rraxff:$rraweek:$rrarows

	rrdtool update "$rrdfile" $rrdcached N:\
$mtxrWlApClientCount

fi


# mtxrHealth

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'
# MikroTik doesn't have mandatory values here so snmpwalk is a must
# no retry (-r 0), do not resend request if OID is missing (-Cf)
result=($(snmpwalk -Cc -m '' -v 1 -r 0 -c $snmprcommunity -Onqt $address \
.1.3.6.1.4.1.14988.1.1.3 \
))

snmpget_status=$?
#test $snmpget_status -eq 0 || exit

IFS=$oldifs


# Undefined is better than 0 for absolute values
mtxrHlVoltage=U
mtxrHlTemperature=U
mtxrHlProcessorFrequency=U

# ugly sanitizing
for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do
	case ${result[oid_row]::29} in
	".1.3.6.1.4.1.14988.1.1.3.8.0 ")
		mtxrHlVoltage=${result[$oid_row]#.1.3.6.1.4.1.14988.1.1.3.8.0 }
		is_number "$mtxrHlVoltage" || mtxrHlVoltage=U
		;;
	".1.3.6.1.4.1.14988.1.1.3.10.0")
		mtxrHlTemperature=${result[$oid_row]#.1.3.6.1.4.1.14988.1.1.3.10.0 }
		is_number "$mtxrHlTemperature" || mtxrHlTemperature=U
		;;
	".1.3.6.1.4.1.14988.1.1.3.14.0")
		mtxrHlProcessorFrequency=${result[$oid_row]#.1.3.6.1.4.1.14988.1.1.3.14.0 }
		is_number "$mtxrHlProcessorFrequency" || mtxrHlProcessorFrequency=U
		;;
	esac
done

# debugging
#echo "$(date '+%F %X') $(basename $0) mtxrHlVoltage=$mtxrHlVoltage mtxrHlTemperature=$mtxrHlTemperature mtxrHlProcessorFrequency=$mtxrHlProcessorFrequency"

# check if we got at last one value
if test $mtxrHlVoltage != U -o $mtxrHlTemperature != U \
		-o $mtxrHlProcessorFrequency != U
then
	rrdfile="$rrddestdir/mtxrHealth.rrd"

	# --start is not necessary
	test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
	DS:Voltage:GAUGE:$rrdheartbeat:0:U \
	DS:Temperature:GAUGE:$rrdheartbeat:U:U \
	DS:ProcessorFrequency:GAUGE:$rrdheartbeat:0:U \
	RRA:AVERAGE:$rraxff:1:$rrarows \
	RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
	RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
	RRA:AVERAGE:$rraxff:$rraday:$rrarows \
	RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
	RRA:MIN:$rraxff:1:$rrarows \
	RRA:MIN:$rraxff:$rrathirtym:$rrarows \
	RRA:MIN:$rraxff:$rratwoh:$rrarows \
	RRA:MIN:$rraxff:$rraday:$rrarows \
	RRA:MIN:$rraxff:$rraweek:$rrarows \
	RRA:MAX:$rraxff:1:$rrarows \
	RRA:MAX:$rraxff:$rrathirtym:$rrarows \
	RRA:MAX:$rraxff:$rratwoh:$rrarows \
	RRA:MAX:$rraxff:$rraday:$rrarows \
	RRA:MAX:$rraxff:$rraweek:$rrarows \
	RRA:LAST:$rraxff:1:$rrarows \
	RRA:LAST:$rraxff:$rrathirtym:$rrarows \
	RRA:LAST:$rraxff:$rratwoh:$rrarows \
	RRA:LAST:$rraxff:$rraday:$rrarows \
	RRA:LAST:$rraxff:$rraweek:$rrarows

	rrdtool update "$rrdfile" $rrdcached N:\
$mtxrHlVoltage:\
$mtxrHlTemperature:\
$mtxrHlProcessorFrequency

fi

# store mtxrHlVoltage to SQL for high voltage reporting
if test $mtxrHlVoltage != U; then

	echo "insert mibmikrotik (agent, time, mtxrHlVoltage) \
values ($agent, unix_timestamp(), $mtxrHlVoltage) \
on duplicate key update \
time = unix_timestamp(), mtxrHlVoltage = $mtxrHlVoltage \
;" >&3

fi

# pass the exit status of the last snmp* command to the collector.sh
exit $snmpget_status
