#!/bin/bash
# Copyright 2009, 2010, 2013 - 2015 Michal Žejdl
# aktualizuje tabulku peer

confdir=$(dirname $0)/../core
config=$confdir/defaults.sh
test -f $config || exit
. $config

# nejdříve si uděláme podmnožinu z tabulky fdb - tabulku freshfdb,
# kde budou pouze záznamy z poslední procházky po přepínačích;
# při tom se zbavujeme matoucích záznamů, kdy přepínač hlásí vlastní mac
# na nějakém svém portu
echo vytvářím tabulku freshfdb
# avoid lastsnmpread=0 switches (awaiting for the first SNMP read)
# thanks to Michal Zaschke
mysql -e "truncate freshfdb; insert into freshfdb select switch, port, mac from fdb left join checked using (mac) where time >= (select min(lastsnmpread) from checked where live = 'y' and hasfdb = 'y' and lastsnmpread > 0) and switch != dns" $modify_opts
echo hotovo, $(mysql -e "select count(*) from freshfdb" $modify_opts -s) záznamů

# teď z tabulky freshfdb uděláme další podmnožinu - tabulku fdbonly,
# která obsahuje trojice local switch (dns), port, remote switch (dns)
echo vytvářím tabulku fdbonly
mysql -e "truncate fdbonly; insert fdbonly select switch, port, dns from freshfdb left join checked using (mac) where hasfdb = 'y'" $modify_opts
echo hotovo, $(mysql -e "select count(*) from fdbonly" $modify_opts -s) záznamů

time=$(date +%s)

# dokud nebude tabulka fdbonly prázdná
while test $(mysql -e "select count(*) from fdbonly" $modify_opts -s) -gt 0; do

	# very ugly but in need to hurry nana stop
	test -e $rundir/topology.sem || exit

  # ladící výpis
  #echo počet nezpracovaných záznamů: $(mysql -e "select count(*) from fdbonly" $modify_opts -s)

  # vybereme řádek s nejmenším počtem local, port (předposlední přepínač
  # na cestě od kořene); ideálně je počet = 1
  pole=($(mysql -e "select * from fdbonly group by local, port order by count(*) limit 1" $modify_opts -s))

  # provedeme zjištění, zda jsou sousedé (vrácená hodnota je vzdálený port nebo nic)
  # chybové výpisy (nejsou sousedé) nás nezajímají
  remoteport=$($netldbdir/remoteport.sh ${pole[*]} 2>/dev/null)

  # tohle je pro lepší čitelnost v následujících dotazech
  local=${pole[0]}
  port=${pole[1]}
  remote=${pole[2]}

  # ladící výpis
  #echo zkoumáme $local $port $remote

  # pokud příkaz něco vrátil...
  if test $remoteport; then

    # ladící výpis
    #echo vkládáme $local $port $remote

    # on duplicate key update asi není to pravé kokosové se třemi unikátními klíči;
    # když ale udělám update všech hodnot, není to nakonec jedno?
    if mysql -e "insert peer values ('$local', $port, $remoteport, '$remote', $time) on duplicate key update switcha = '$local', porta = $port, portb = $remoteport, switchb = '$remote', time = $time; insert peer values ('$remote', $remoteport, $port, '$local', $time) on duplicate key update switcha = '$remote', porta = $remoteport, portb = $port, switchb = '$local', time = $time" $modify_opts; then

      # došlo k chybě i přes on duplicate key update - změna topologie, chyba v algoritmu, jinak naučené MAC...

      # takže máme správnou čtveřici a smažeme z peer všechno, co překáží, pokud to tam tedy je
      # vycházím ze tří unikátních klíčů nad tabulkou peer
      mysql -e "delete from peer where switcha = '$local' and porta = $port or switchb = '$local' and portb = $port or switcha = '$remote' and porta = $remoteport or switchb = '$remote' and portb = $remoteport or switcha = '$local' and switchb = '$remote' or switchb = '$local' and switcha = '$remote'" $modify_opts

      # a vložíme znovu, teď už by neměl být duplicitní klíč
      mysql -e "insert peer values ('$local', $port, $remoteport, '$remote', $time); insert peer values ('$remote', $remoteport, $port, '$local', $time)" $modify_opts
    fi

    # smažeme všechny remote - pokud jsme nalezli sousedy, nemají záznamy
    # o výskytu remote na jiných přepínačích význam - pokud ano,
    # je stejně špatně celý algoritmus a sousedy bude třeba získat jinak
    mysql -e "delete from fdbonly where remote = '$remote'" $modify_opts
  else
    
    # smažeme pouze zkoumaný řádek; možná by se dal smazat i reciproký záznam (remote, *, local)
    mysql -e "delete from fdbonly where local = '$local' and port = $port and remote = '$remote'" $modify_opts
  fi

  # aby šel cyklus přerušit
  usleep 1
done
